/*******************************************************************************
 * Fraco - Eclipse plug-in to detect fragile comments
 *
 * Copyright (C) 2019 McGill University
 *     
 * Eclipse Public License - v 2.0
 *
 *  THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS ECLIPSE
 *  PUBLIC LICENSE v2.0. ANY USE, REPRODUCTION OR DISTRIBUTION
 *  OF THE PROGRAM CONSTITUTES RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT.
 *******************************************************************************/
/**
 * 
 */

package ca.mcgill.cs.stg.fraco.Properties.handlers;

import org.eclipse.core.filebuffers.FileBuffers;
import org.eclipse.core.filebuffers.ITextFileBuffer;
import org.eclipse.core.filebuffers.ITextFileBufferManager;
import org.eclipse.core.filebuffers.LocationKind;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.FindReplaceDocumentAdapter;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IMarkerResolution2;

import ca.mcgill.cs.stg.fraco.AstBuilder;
import ca.mcgill.cs.stg.fraco.DetectFragileComments;
import ca.mcgill.cs.stg.fraco.rules.CommonRules;

/**
 * @author inder
 *
 */
public class QuickFix implements IMarkerResolution2 {

	IMarker marker;
	int lineNumber = 0;
	private IDocument doc;
	private IResource resource;
	private boolean caseSensitive = true;
	private boolean wholeWord = true;
	public static CommonRules commonRules = new CommonRules();

	public QuickFix() {
	}

	@Override
	public String getLabel() {
		return "Replace old identifier with new one.";
	}

	@Override
	public void run(final IMarker marker) {
		this.marker = marker;
		getCompilationUnit(marker);
		try {
			if (DetectFragileComments._markerIds.contains(marker)) {
				int index = DetectFragileComments._markerIds.indexOf(marker);
				DetectFragileComments._markerIds.remove(index);
			}
			marker.delete();

		} catch (CoreException e) {
			e.printStackTrace();
		}
	}

	private void getCompilationUnit(IMarker marker) {
		this.lineNumber = marker.getAttribute(IMarker.LINE_NUMBER, 0);
		resource = marker.getResource();
		if (resource instanceof IFile && resource.isAccessible()) {
			IFile file = (IFile) resource;
			ICompilationUnit cu = (ICompilationUnit) JavaCore.create(file);
			CompilationUnit astRoot = AstBuilder.parse(cu);
			getTextBuffer(astRoot);
		}
	}

	private void findIdentifierInDocument() {
		FindReplaceDocumentAdapter search = new FindReplaceDocumentAdapter(this.doc);
		try {
			String wordToFind = marker.getAttribute("key", "");
			String wordToReplace = marker.getAttribute("value", "");
			String originalIdentifier = marker.getAttribute("original", "");
			if (!originalIdentifier.equalsIgnoreCase(wordToFind)) {
				wordToReplace = commonRules.convertSingularToPlural(wordToReplace);
			}
			int partitionLength = marker.getAttribute("length", 100);
			// setCaseSensitiveFlag(idType);
			IRegion found = search.find(this.lineNumber, wordToFind, true, caseSensitive, wholeWord, false); // look
																												// at
																												// the
																												// parameters
																												// for
																												// this
																												// one

			if (!replaceTheWord(found, wordToFind, wordToReplace)) {
				// for @see tags
				found = search.find(this.lineNumber, wordToFind + "(", true, caseSensitive, false, false);
				if (!replaceTheWord(found, wordToFind, wordToReplace))
					found = search.find(this.lineNumber, wordToFind + ".", true, caseSensitive, false, false);
				replaceTheWord(found, wordToFind, wordToReplace);
			}
		} catch (BadLocationException e) {
			System.out.println(e.getMessage());
		}
	}

	private void setCaseSensitiveFlag(String type) {
		if (type.equalsIgnoreCase("class")) {
			caseSensitive = true;
		} else {
			caseSensitive = false;
		}
	}

	private boolean replaceTheWord(IRegion found, String find, String replace) {
		try {
			if (found != null) {
				doc.replace(found.getOffset(), find.length(), replace);
				// String documentText=doc.get();
				resource.refreshLocal(3, new NullProgressMonitor());
				return true;
			}
			return false;
		} catch (BadLocationException e) {
			e.printStackTrace();
		} catch (CoreException e) {
			e.printStackTrace();
		}
		return false;
	}

	public void getTextBuffer(CompilationUnit unit) {
		ITextFileBufferManager bufferManager = FileBuffers.getTextFileBufferManager();
		IPath path = unit.getJavaElement().getPath();
		try {
			try {
				bufferManager.connect(path, LocationKind.IFILE, null);
				ITextFileBuffer textFileBuffer = bufferManager.getTextFileBuffer(path, LocationKind.IFILE);
				// retrieve the buffer
				doc = textFileBuffer.getDocument();
				// ... edit the document here ...
				findIdentifierInDocument();
				// commit changes to underlying file
				textFileBuffer.commit(null /* ProgressMonitor */, false /* Overwrite */);
				bufferManager.disconnect(path, null);
			} catch (CoreException e) {
				e.printStackTrace();
			}
		} finally {

		}
	}

	@Override
	public String getDescription() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Image getImage() {
		// TODO Auto-generated method stub
		return null;
	}

}
