/*******************************************************************************
 * Fraco - Eclipse plug-in to detect fragile comments
 *
 * Copyright (C) 2019 McGill University
 *     
 * Eclipse Public License - v 2.0
 *
 *  THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS ECLIPSE
 *  PUBLIC LICENSE v2.0. ANY USE, REPRODUCTION OR DISTRIBUTION
 *  OF THE PROGRAM CONSTITUTES RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT.
 *******************************************************************************/

package ca.mcgill.cs.stg.fraco.parsers;

import org.eclipse.jdt.core.dom.EnumConstantDeclaration;
import org.eclipse.jdt.core.dom.FieldAccess;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;

import ca.mcgill.cs.stg.fraco.AstBuilder;
import ca.mcgill.cs.stg.fraco.utils.Type;
import ca.mcgill.cs.stg.fraco.visitors.FieldVisitor;

public class FieldParser extends ParseAstNodes {

	public void parseFieldDeclarations(FieldVisitor visitor) {
		for (FieldDeclaration field : visitor.getFieldDeclarations()) {
			String fieldName = ((VariableDeclarationFragment) field.fragments().get(0)).getName().getIdentifier();
			int mod = field.getModifiers();
			int index = AstBuilder._currentCompilationUnit.firstLeadingCommentIndex(field);

			String parentName = getClassName(field.getParent());
			int classModifier = getClassModifier(parentName, field.getParent());
			if (AstBuilder._renamedElement != null && fieldName.equals(AstBuilder._renamedElement.getElementName())) {
				AstBuilder._identifier.set_type(Type.FIELD);
				AstBuilder._identifier.set_modifier(mapModifier(mod));
				AstBuilder._identifier.set_parentModifier(mapModifier(classModifier));
				// _identifier.set_cuName(_currentCompilationUnit.getJavaElement().getElementName());
				if (traverseLeadingComments(index))
					break;
			}
		}
	}

	public void parseEnumConstants(FieldVisitor visitor) {
		for (EnumConstantDeclaration field : visitor.getEnumConstants()) {
			String fieldName = field.getName().getIdentifier();
			int mod = field.getModifiers();
			if (mod == 0)
				mod = 1;
			int index = AstBuilder._currentCompilationUnit.firstLeadingCommentIndex(field);

			String parentName = getClassName(field.getParent());
			int classModifier = getClassModifier(parentName, field.getParent());
			if (AstBuilder._renamedElement != null && fieldName.equals(AstBuilder._renamedElement.getElementName())) {
				AstBuilder._identifier.set_type(Type.FIELD);
				AstBuilder._identifier.set_modifier(mapModifier(mod));
				AstBuilder._identifier.set_parentModifier(mapModifier(classModifier));
				traverseLeadingComments(index);
				break;
			}
		}
	}

	public void parseFieldAccess(FieldVisitor visitor) {
		for (FieldAccess field : visitor.getFieldAccesses()) {
			String fieldName = field.getName().toString();
			int index = AstBuilder._currentCompilationUnit.firstLeadingCommentIndex(field);
			if (AstBuilder._renamedElement != null
					&& fieldName.equals(AstBuilder._renamedElement.getElementName())) {
				AstBuilder._identifier.set_type(Type.FIELD);
				traverseLeadingComments(index);
			}
		}
	}

}
