/*******************************************************************************
 * Fraco - Eclipse plug-in to detect fragile comments
 *
 * Copyright (C) 2019 McGill University
 *     
 * Eclipse Public License - v 2.0
 *
 *  THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS ECLIPSE
 *  PUBLIC LICENSE v2.0. ANY USE, REPRODUCTION OR DISTRIBUTION
 *  OF THE PROGRAM CONSTITUTES RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT.
 *******************************************************************************/
package ca.mcgill.cs.stg.fraco.visitors;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.*;

/**
 * ClassVisitor is used to get all the class declarations, references, enums and
 * type declarations from AST.
 * 
 * @author inder
 *
 */
public class ClassVisitor extends ASTVisitor {
	List<ClassInstanceCreation> classInstances = new ArrayList<ClassInstanceCreation>();
	List<AnonymousClassDeclaration> anonymousClasses = new ArrayList<AnonymousClassDeclaration>();
	List<TypeDeclaration> classdeclaration = new ArrayList<TypeDeclaration>();
	List<EnumDeclaration> enumdeclaration = new ArrayList<EnumDeclaration>();
	List<AnnotationTypeDeclaration> interfacedeclaration = new ArrayList<AnnotationTypeDeclaration>();

	/*
	 * Visits every class instance creation node in AST
	 * 
	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.
	 * ClassInstanceCreation)
	 */
	@Override
	public boolean visit(ClassInstanceCreation node) {

		classInstances.add(node);
		return super.visit(node);
	}

	/**
	 * @return the list of all class instance creation nodes
	 */
	public List<ClassInstanceCreation> getClassInstances() {
		return classInstances;
	}

	/*
	 * Visits all the anonymous class declaration nodes in AST
	 * 
	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.
	 * AnonymousClassDeclaration)
	 */
	@Override
	public boolean visit(AnonymousClassDeclaration node) {
		anonymousClasses.add(node);
		return super.visit(node);
	}

	/**
	 * @return the list of all anonymous class declaration nodes
	 */
	public List<AnonymousClassDeclaration> getAnonymousClasses() {
		return anonymousClasses;
	}

	/*
	 * Visits all the type declaration nodes in AST
	 * 
	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.
	 * TypeDeclaration)
	 */
	@Override
	public boolean visit(TypeDeclaration node) {
		classdeclaration.add(node);
		return super.visit(node);
	}

	/**
	 * @return the list of all type declaration nodes
	 */
	public List<TypeDeclaration> getClasses() {
		return classdeclaration;
	}

	/*
	 * Visits all the enum declaration nodes in AST
	 * 
	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.
	 * EnumDeclaration)
	 */
	@Override
	public boolean visit(EnumDeclaration node) {
		enumdeclaration.add(node);
		return super.visit(node);
	}

	/**
	 * @return the list of all enum declaration nodes
	 */
	public List<EnumDeclaration> getEnums() {
		return enumdeclaration;
	}

	/*
	 * Visits all the enum declaration nodes in AST
	 * 
	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.
	 * EnumDeclaration)
	 */
	@Override
	public boolean visit(AnnotationTypeDeclaration node) {
		interfacedeclaration.add(node);
		return super.visit(node);
	}

	/**
	 * @return the list of all enum declaration nodes
	 */
	public List<AnnotationTypeDeclaration> getInterfaces() {
		return interfacedeclaration;
	}
}
