/*******************************************************************************
 * Fraco - Eclipse plug-in to detect fragile comments
 *
 * Copyright (C) 2019 McGill University
 *     
 * Eclipse Public License - v 2.0
 *
 *  THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS ECLIPSE
 *  PUBLIC LICENSE v2.0. ANY USE, REPRODUCTION OR DISTRIBUTION
 *  OF THE PROGRAM CONSTITUTES RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT.
 *******************************************************************************/
package ca.mcgill.cs.stg.fraco.visitors;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.EnumConstantDeclaration;
import org.eclipse.jdt.core.dom.FieldAccess;
import org.eclipse.jdt.core.dom.FieldDeclaration;

/**
 * FieldVisitor is used to traverse all the field nodes in an AST.
 * 
 * @author inder
 *
 */
public class FieldVisitor extends ASTVisitor {

	List<FieldDeclaration> fieldDeclarations = new ArrayList<FieldDeclaration>();
	List<FieldAccess> fieldAccesses = new ArrayList<FieldAccess>();
	List<EnumConstantDeclaration> enumConstants = new ArrayList<EnumConstantDeclaration>();

	/*
	 * (non-Javadoc) Traverses all the field declaration nodes in AST.
	 * 
	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.
	 * FieldDeclaration)
	 */
	@Override
	public boolean visit(FieldDeclaration node) {

		fieldDeclarations.add(node);
		return super.visit(node);
	}

	/**
	 * @return the list of field declaration nodes
	 */
	public List<FieldDeclaration> getFieldDeclarations() {
		return fieldDeclarations;
	}

	/*
	 * (non-Javadoc) Traverses all the field access nodes in AST.
	 * 
	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.
	 * FieldAccess)
	 */
	@Override
	public boolean visit(FieldAccess node) {
		fieldAccesses.add(node);
		return super.visit(node);
	}

	/**
	 * @return the list of all field access nodes in AST
	 */
	public List<FieldAccess> getFieldAccesses() {
		return fieldAccesses;
	}

	/*
	 * (non-Javadoc) Traverses all the enum declaration nodes in AST.
	 * 
	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.
	 * EnumConstantDeclaration)
	 */
	@Override
	public boolean visit(EnumConstantDeclaration node) {
		enumConstants.add(node);
		return super.visit(node);
	}

	/**
	 * @return the list of all enum declarations in AST.
	 */
	public List<EnumConstantDeclaration> getEnumConstants() {
		return enumConstants;
	}

}