/*******************************************************************************
 * Fraco - Eclipse plug-in to detect fragile comments
 *
 * Copyright (C) 2019 McGill University
 *     
 * Eclipse Public License - v 2.0
 *
 *  THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS ECLIPSE
 *  PUBLIC LICENSE v2.0. ANY USE, REPRODUCTION OR DISTRIBUTION
 *  OF THE PROGRAM CONSTITUTES RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT.
 *******************************************************************************/
package ca.mcgill.cs.stg.fraco.visitors;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.eclipse.jdt.core.dom.VariableDeclarationStatement;

/**
 * LocalVariableVisitor is used to traverse all the local variable nodes in an
 * AST.
 * 
 * @author inder
 *
 */
public class LocalVariableVisitor extends ASTVisitor {

	List<SingleVariableDeclaration> variableDeclarations = new ArrayList<SingleVariableDeclaration>();
	List<VariableDeclarationStatement> variableStatements = new ArrayList<VariableDeclarationStatement>();

	/*
	 * (non-Javadoc) Traverse all the local variable declaration nodes. These
	 * include the declaration of formal parameters of a method as well.
	 * 
	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.
	 * SingleVariableDeclaration)
	 */
	@Override
	public boolean visit(SingleVariableDeclaration node) {
		variableDeclarations.add(node);
		return super.visit(node);
	}

	/**
	 * @return the list of all local variables and formal parameters
	 *         declarations.
	 */
	public List<SingleVariableDeclaration> getVariableDeclarations() {
		return variableDeclarations;
	}

	/*
	 * (non-Javadoc) Traverses all variable declaration statements in AST.
	 * 
	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.
	 * VariableDeclarationStatement)
	 */
	@Override
	public boolean visit(VariableDeclarationStatement node) {
		variableStatements.add(node);
		return super.visit(node);
	}

	/**
	 * @return the list of all variable declaration statements which are used to
	 *         retrieve the local variables.
	 */
	public List<VariableDeclarationStatement> getVariableStatements() {
		return variableStatements;
	}

}
